#include "slideobject.h"

#include "slideworld.h"
#include "slideface.h"

//////////////////////////////////////////////////////////////////////
// CSLIDEObject Class
//

CSLIDEObject::CSLIDEObject()
{
  m_plistFaceIDs = NULL;
}

CSLIDEObject::~CSLIDEObject()
{
}

VOID CSLIDEObject::Init(CHAR *pcFile, UINT_32 uiLine, UINT_32 uiColumn, 
			CHAR *pcID,
			CListInt *plistFaceIDs)
{
  CSLIDEPrimitive::Init(pcFile, uiLine, uiColumn, pcID);

  m_plistFaceIDs = plistFaceIDs;
  m_listCSLIDEFace.Init();
}

VOID CSLIDEObject::Uninit()
{
  CSLIDEPrimitive::Uninit();

  if ( m_plistFaceIDs != NULL )
    {
      delete m_plistFaceIDs;
      m_plistFaceIDs = NULL;
    }
}

BOOL CSLIDEObject::Initialized()
{
  if ( !CSLIDEPrimitive::Initialized() )
    {
      return FALSE;
    }

  return (m_plistFaceIDs != NULL);
}

BOOL CSLIDEObject::CheckSemantics(CSLIDEWorld *pCSLIDEWorld)
{
  BOOL bCorrect;
  CListIterInt iter;
  CHAR *pcID;
  CSLIDEFace *pCSLIDEFace;

  if ( !CSLIDEPrimitive::CheckSemantics(pCSLIDEWorld) )
    {
      return FALSE;
    }

  bCorrect = TRUE;

  // Check Face References

  iter.Init(m_plistFaceIDs);

  for ( pcID = (CHAR*)(iter.PeekFirst());
	pcID != NULL;
	pcID = (CHAR*)(iter.PeekNext()) )
    {
      pCSLIDEFace = pCSLIDEWorld->PeekFace(pcID);
      if ( pCSLIDEFace == NULL )
	{
	  HandleError("Undefined %s \"%s\"", gcppcSLIDEReservedWords[sliderwtFace], pcID);
	  bCorrect = FALSE;
	}
      m_listCSLIDEFace.InsertLast(pCSLIDEFace, 0);
    }

  return bCorrect;
}

BOOL CSLIDEObject::Valid()
{
  if ( !CSLIDEPrimitive::Valid() )
    {
      return FALSE;
    }

  return TRUE;
}

CSLIDEPrimitiveIter *CSLIDEObject::AllocateIter()
{
  CSLIDEObjectIter *pCSLIDEObjectIter;

  pCSLIDEObjectIter = new CSLIDEObjectIter;
  ASSERT(pCSLIDEObjectIter != NULL );
  pCSLIDEObjectIter->Init(this);

  return pCSLIDEObjectIter;
}

VOID CSLIDEObject::FreeIter(CSLIDEPrimitiveIter *pCSLIDEObjectIter)
{
  delete pCSLIDEObjectIter;
}

///////////////////////////////////////////////////
// START: ASSIGNMENT 2
// 

// Render the object geometry
VOID CSLIDEObject::Render()
{
  CListIterInt iter;
  CSLIDEFace *pCSLIDEFace;
  
  iter.Init(&m_listCSLIDEFace);
  
  // Fix this
}

// This should return the first point in the object that is sufficiently
// close to the mouse cursor or NULL if there is no vertex near the mouse 
// cursor
CSLIDEPoint *CSLIDEObject::SelectPoint(FLOAT fX, FLOAT fY)
{
  CListIterInt iter;
  CSLIDEPoint *pCSLIDEPoint;
  CSLIDEFace *pCSLIDEFace;

  iter.Init(&m_listCSLIDEFace);

  // Fix this

  return NULL;
}

// Write the new SLIDE geometry description -- NO MORPHING
VOID CSLIDEObject::WriteSLIDE(FILE *fpOutputFile)
{
  CListIterInt iter;
  CSLIDEFace *pCSLIDEFace;

  ASSERT( fpOutputFile != NULL );

  fprintf(fpOutputFile, "%s %s ( ", 
	  gcppcSLIDEReservedWords[sliderwtObject], m_pcID);

  iter.Init(&m_listCSLIDEFace);

  for ( pCSLIDEFace = (CSLIDEFace*)iter.PeekFirst(); 
	pCSLIDEFace != NULL;
	pCSLIDEFace = (CSLIDEFace*)iter.PeekNext() )
    {
      fprintf(fpOutputFile, "%s ", pCSLIDEFace->GetID());      
    }

  fprintf(fpOutputFile, ")\n");


  fprintf(fpOutputFile, "  %s %s\n",
	  gcppcSLIDEReservedWords[sliderwtShading], 
	  gcppcSLIDEShadingReservedWords[slideshtSLF_WIRE]);
  fprintf(fpOutputFile, "  %s %s\n",
	  gcppcSLIDEReservedWords[sliderwtSolid], 
	  gcppcSLIDESolidReservedWords[slidestSLF_HOLLOW]);

  fprintf(fpOutputFile, "%s\n", 
	  gcppcSLIDEReservedWords[sliderwtEndObject]);
}

// Write the MORPHING SLIDE geometry description
VOID CSLIDEObject::WriteSLIDEMorph(FILE *fpOutputFile)
{
  // Fix this
}

// 
// END  : ASSIGNMENT 2
///////////////////////////////////////////////////

UINT_32 CSLIDEObject::GetFaceCount()
{
  return m_listCSLIDEFace.GetLength();
}

//////////////////////////////////////////////////////////////////////////////
// CSLIDEObjectIter Class
//

CSLIDEObjectIter::CSLIDEObjectIter()
{
}

CSLIDEObjectIter::~CSLIDEObjectIter()
{
}

VOID CSLIDEObjectIter::Init(CSLIDEObject *pCSLIDEObject)
{
  CSLIDEPrimitiveIter::Init(pCSLIDEObject);

  m_iter.Init(&(pCSLIDEObject->m_listCSLIDEFace));
}

VOID CSLIDEObjectIter::Uninit()
{
  CSLIDEPrimitiveIter::Uninit();
}

CSLIDEFace *CSLIDEObjectIter::PeekFirstCSLIDEFace()
{
  return (CSLIDEFace*)(m_iter.PeekFirst());
}

CSLIDEFace *CSLIDEObjectIter::PeekNextCSLIDEFace()
{
  return (CSLIDEFace*)(m_iter.PeekNext());
}
